using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.Resources;
using System.Reflection;
using System.Threading;
using System.Globalization;

namespace RTGA
{
	/// <summary>
	/// Class implementing the computing and drawing of the step response
	/// Mihai Virtosu, virtosumihai@yahoo.com
	/// </summary>
	public class DrawingForm : System.Web.UI.Page
	{
		#region Private Methods
		
		/// <summary>
		/// Method that will get the localizable strings from the resource files
		/// </summary>
		void GetLocaleStrings()
		{
			ResourceManager resMan = new ResourceManager("RTGA.DrawingForm",
				Assembly.GetExecutingAssembly());

			strErrorQueryString = resMan.GetString("ErrorQueryString");
			strTitleDrawing = resMan.GetString("TitleDrawing");
			strTitleGenetic = resMan.GetString("TitleGeneticResponse");
			strTitleKessler = resMan.GetString("TitleKesslerResponse");
		}

		/// <summary>
		/// Method that will draw the closed loop response for both the genetic algorithm
		/// and the Kessler (classical) algorithm
		/// </summary>
		void DrawResponse()
		{
			//Sizes of the plotted response
			int xOutSize = 480;
			int yOutSize = 620;
			int xInnerSize = 420;
			int yInnerSize = 420;

			//Objects to handle drawing
			Bitmap img = new Bitmap(480,620);
			Graphics g = Graphics.FromImage(img);

			//Fill the background of the drawing
			g.FillRectangle(System.Drawing.Brushes.LightGray,0,0,xOutSize,yOutSize);
			g.FillRectangle(System.Drawing.Brushes.White,30,30,xInnerSize,yInnerSize);

			//Get the required parameters' values from the session object
			double[,] y = (double[,])Session["y"];
			double[,] yKessler = (double[,])Session["yKessler"];
			double[,] t = (double[,])Session["t"];
			double[,] tKessler = (double[,])Session["tKessler"];

			int length = t.Length;
			double Tfinal = t[length-1,0];

			length = tKessler.Length;
			double TKFinal = tKessler[length-1,0];

			double c = (double)Session["c2"];
			double tt = (double)Session["tt2"];

			double maxTime = 4 * tt;
			if (4*8*c > maxTime)
				maxTime = (double)4*8*c; //8*c = timpul tranzitoriu pentru Kessler
	
			if(maxTime > Tfinal)
				maxTime = Tfinal;

			if(maxTime > TKFinal)
				maxTime = TKFinal;

			//Prepare the font and the brushes
			Font font = new Font("Arial",10);
			SolidBrush redBrush = new SolidBrush(Color.Red);
			SolidBrush blackBrush = new SolidBrush(Color.Black);
			SolidBrush greenBrush = new SolidBrush(Color.Green);
				
			//Draw the x-axis
			Pen redPen = new Pen(redBrush,1);
			Pen blackPen = new Pen(blackBrush,1);
			Pen greenPen = new Pen(greenBrush,1);
			for(int i=1;i<=5;i++)
			{
				string label = "";
				if( (int)(maxTime/5) < 1 )
				{	
					try
					{
						//Retain only the first 3 decimals for the labels
						label=Convert.ToString(i*(double)(maxTime/5)).Substring(0,3);
					}
					catch
					{
						Page.RegisterClientScriptBlock("key1",
							"<SCRIPT>alert('" + Convert.ToString(i*(double)(maxTime/5)) +
							"');</SCRIPT>");
					}
				}
				else
					label=Convert.ToString(i*(int)(maxTime/5));
												
				//Draw the label
				g.DrawString(label,font,blackBrush,(float)(420/maxTime*i*maxTime/5+25),455);
					
				//Draw the ticks
				g.DrawLine(blackPen,(float)(420/maxTime*i*maxTime/5+30),450,(float)(420/maxTime*i*maxTime/5+30),445);
			}

			//Plot the genetic algorithm response graph
			for(int i=0;i<y.Length-1;i++)
			{
				if(t[i+1,0]<maxTime)
					g.DrawLine(redPen,(float)(30 + 420*t[i,0]/maxTime),(float)(450-y[i,0]*200), (float)(30 + 420*t[i+1,0]/maxTime),(float)(450-y[i+1,0]*200));
			}

			//Plot the Kessler method response graph
			for(int i=0;i<yKessler.Length-1;i++)
			{
				if(tKessler[i+1,0]<maxTime)
					g.DrawLine(greenPen,(float)(30 + 420*tKessler[i,0]/maxTime),(float)(450-yKessler[i,0]*200), (float)(30 + 420*tKessler[i+1,0]/maxTime),(float)(450-yKessler[i+1,0]*200));
			}

			//Draw the y-axis labels and ticks
			g.DrawString("1.0",font,blackBrush,5,243);
			g.DrawLine(blackPen,30,250,33,250);

			g.DrawString("0.5",font,blackBrush,5,343);
			g.DrawLine(blackPen,30,350,33,350);

			g.DrawString("1.5",font,blackBrush,5,143);
			g.DrawLine(blackPen,30,150,33,150);

			g.DrawString("2.0",font,blackBrush,5,43);
			g.DrawLine(blackPen,30,50,33,50);

			//Draw the caption for the plotting
			Font bigFont = new Font("Arial",12,FontStyle.Bold);
			g.DrawString(strTitleDrawing,bigFont,blackBrush, 85, 500);

			g.DrawLine(redPen,100,550,125,550);
			g.DrawString(strTitleGenetic,font,blackBrush,135,543);

			g.DrawLine(greenPen,100,570,125,570);
			g.DrawString(strTitleKessler,font,blackBrush,135,563);

			//Save the drawing to the output stream
			Response.ContentType="image/JPEG";
			img.Save(Response.OutputStream,ImageFormat.Jpeg);
		}
		#endregion

		#region Private Members
		
		//Genetic algorithm parameters
		private double[,] a =  new double[1,1];
		private double[,] b =  new double[1,1];
		private double[,] c =  new double[1,1];
		private double[,] Kp =  new double[1,1];
		private double[,] MAXGEN = new double[1,1];
		private double[,] Chrom_in = new double[1,1];
		private double[,] CSreg=new double[1,1];
		private double[,] CTt=new double[1,1];
		private double[,] minKr=new double[1,1];
		private double[,] minTi=new double[1,1];
		private double[,] minTd=new double[1,1];
		private double[,] maxKr=new double[1,1];
		private double[,] maxTi=new double[1,1];
		private double[,] maxTd=new double[1,1];
		private double[,] inTd=new double[1,1];
		private double[,] inTi=new double[1,1];
		private double[,] inKr=new double[1,1];

		//Localizable strings
		private string strErrorQueryString;
		private string strTitleDrawing;
		private string strTitleGenetic;
		private string strTitleKessler;

		#endregion

		#region Load Event
		
		/// <summary>
		/// Event handler for the Page Load event.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!IsPostBack)
			{
				// Get the user's perferred language.
				string sLang = Request.UserLanguages[0];
				// Set the thread's culture for formatting, comparisons, etc.
				Thread.CurrentThread.CurrentCulture = 
					CultureInfo.CreateSpecificCulture(sLang);
				// Set the thread's UICulture to load resources 
				// from satellite assembly.
				Thread.CurrentThread.CurrentUICulture = new CultureInfo(sLang);
			}

			// Get locale strings
			GetLocaleStrings();

			// If this page has been caled with parameters
			if(Request.QueryString["a"] != null)
			{
				// Store the values in the session object
				try
				{
					Session["a"] = XmlConvert.ToDouble(Request.QueryString["a"]);
					Session["b"] = XmlConvert.ToDouble(Request.QueryString["b"]);
					Session["c"] = XmlConvert.ToDouble(Request.QueryString["c"]);
					Session["Kp"] = XmlConvert.ToDouble(Request.QueryString["Kp"]);
					Session["MAXGEN"] = XmlConvert.ToDouble(Request.QueryString["MAXGEN"]);
					Session["CTt"] = XmlConvert.ToDouble(Request.QueryString["CTt"]);
					Session["minKr"] = XmlConvert.ToDouble(Request.QueryString["minKr"]);
					Session["minTi"] = XmlConvert.ToDouble(Request.QueryString["minTi"]);
					Session["minTd"] = XmlConvert.ToDouble(Request.QueryString["minTd"]);
					Session["maxKr"] = XmlConvert.ToDouble(Request.QueryString["maxKr"]);
					Session["maxTi"] = XmlConvert.ToDouble(Request.QueryString["maxTi"]);
					Session["maxTd"] = XmlConvert.ToDouble(Request.QueryString["maxTd"]);
				}
				catch
				{
					Page.RegisterClientScriptBlock("key2", 
						"<SCRIPT>alert('" + strErrorQueryString + "');</SCRIPT>");
				}

				// 14 is the length of the slider's scale, and since CTt and CSreg are
				// complementary, their sum must equal 14
				Session["CSreg"] = 14 - XmlConvert.ToDouble(Request.QueryString["CTt"]);

				// Get the parameters' values
				a[0,0]=(double)Session["a"];
				b[0,0]=(double)Session["b"];
				c[0,0]=(double)Session["c"];
				Kp[0,0]=(double)Session["Kp"];
				CSreg[0,0]=(double)Session["CSreg"];
				CTt[0,0]=(double)Session["CTt"];
				MAXGEN[0,0]=(double)Session["MAXGEN"];
				minKr[0,0]=(double)Session["minKr"];
				minTi[0,0]=(double)Session["minTi"];
				minTd[0,0]=(double)Session["minTd"];
				maxKr[0,0]=(double)Session["maxKr"];
				maxTi[0,0]=(double)Session["maxTi"];
				maxTd[0,0]=(double)Session["maxTd"];

				inKr[0,0] = XmlConvert.ToDouble(Request.QueryString["inKr"]);
				inTi[0,0] = XmlConvert.ToDouble(Request.QueryString["inTi"]);
				inTd[0,0] = XmlConvert.ToDouble(Request.QueryString["inTd"]);

				if(Session["Chrom"] != null && Request.QueryString["NoChromIn"]=="false")
					Chrom_in = (double[,])Session["Chrom"];
				else
					Chrom_in[0,0]=9;	// This is a convention
										// If we pass 9 as a parameter, the imported
										// Matlab function will not use any previous
										// information to compute a new controller

				//Instantiate the object handling the Matlab computing part
				multarg objMultarg = new multarg();

				// Declare the output variables
				double[,]tt=new double[1,1];
				double[,]suprareglaj=new double[1,1];
				double[,]Kr=new double[1,1];
				double[,]Ti=new double[1,1];
				double[,]Td=new double[1,1];
				double[,]y=new double[1,1];
				double[,]t=new double[1,1];
				double[,]Chrom=new double[1,1];
				double[,]tKessler=new double[1,1];
				double[,]yKessler=new double[1,1];

				// Call the Matlab imported function
				objMultarg.csMultArg(ref tt,
					ref suprareglaj,
					ref Kr,
					ref Ti,
					ref Td,
					ref Chrom,
					ref y,
					ref t,
					ref yKessler,
					ref tKessler,
					a,
					b,
					c,
					Kp,
					MAXGEN,
					Chrom_in,
					CSreg,
					CTt,
					minKr,
					minTi,
					minTd,
					maxKr,
					maxTi,
					maxTd,
					inKr,
					inTi,
					inTd
					);

				// Store the output parameters in the seesion object for future use
				Session["Chrom"] = Chrom;
				Session["y"] = y;
				Session["t"]= t;
				Session["yKessler"] = yKessler;
				Session["tKessler"] = tKessler;
				Session["tt2"] = tt[0,0];
				Session["c2"] = c[0,0];
				
				// Show the numeric results in the left pane (the "contents" pane)
				// and the plotted step response in the right pane (the "main" pane)
				Response.Write("<SCRIPT>window.open('MainForm.aspx?tt=" + 
					XmlConvert.ToString(tt[0,0]) + "&suprareglaj=" + 
					XmlConvert.ToString(suprareglaj[0,0]*100) + "&Kr=" + 
					XmlConvert.ToString(Kr[0,0]) + "&Ti=" + 
					XmlConvert.ToString(Ti[0,0]) + "&Td=" +
					XmlConvert.ToString(Td[0,0]) + "&a=" + 
					XmlConvert.ToString(a[0,0]) + "&b=" + 
					XmlConvert.ToString(b[0,0]) + "&c=" + 
					XmlConvert.ToString(c[0,0]) + "&Kp=" + 
					XmlConvert.ToString(Kp[0,0]) + "&MAXGEN=" +
					XmlConvert.ToString(MAXGEN[0,0]) + "&minKr=" +
					XmlConvert.ToString(minKr[0,0]) + "&minTi=" +
					XmlConvert.ToString(minTi[0,0]) + "&minTd=" +
					XmlConvert.ToString(minTd[0,0]) + "&maxKr=" +
					XmlConvert.ToString(maxKr[0,0]) + "&maxTi=" +
					XmlConvert.ToString(maxTi[0,0]) + "&maxTd=" +
					XmlConvert.ToString(maxTd[0,0]) + 
					"&NoChromIn=" + Request.QueryString["NoChromIn"] +
					"&ticks=" + DateTime.Now.Ticks.ToString() +
					"','contents'); window.open('DrawingForm.aspx?draw=true" + "&ticks=" + 
					DateTime.Now.Ticks.ToString() +
					"','main');</SCRIPT>");
			}

			// If this page was called with the "draw" parameter set to "true"
			if(Request.QueryString["draw"] == "true")
			{
				// Begin plotting the time response of the closed-loop system
				DrawResponse();
			}
		}

#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
