using System;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace matlabMatrix
{

   /// <summary>
   /// Interface to mex types from C#
   ///      
   ///      Somewhat incomplete, but should serve most purposes until mathworks makes an official version.
   ///      Free for non-commercial use.  Commercial use requires a $30./developer license.  
   ///      
   ///      Code provided "as-is".  No warranty is made for appropriateness of use to a particular application.
   /// 
   /// (c) Erik Frederick, 2002
   /// erik_frederick@yahoo.com
   /// V1.0
   /// 
   /// </summary>
   /// 

   [ClassInterface(ClassInterfaceType.None)]
   public class mxMatrix
   {

      public enum mxClassID:int
      {
         mxUNKNOWN_CLASS = 0,
         mxCELL_CLASS,
         mxSTRUCT_CLASS,
         mxOBJECT_CLASS,
         mxCHAR_CLASS,
         mxSPARSE_CLASS,
         mxDOUBLE_CLASS,
         mxSINGLE_CLASS,
         mxINT8_CLASS,
         mxUINT8_CLASS,
         mxINT16_CLASS,
         mxUINT16_CLASS,
         mxINT32_CLASS,
         mxUINT32_CLASS,
         mxINT64_CLASS,		/* place holder - future enhancements */
         mxUINT64_CLASS,		/* place holder - future enhancements */
         mxFUNCTION_CLASS,
         mxOPAQUE_CLASS
      };

      public enum mxComplexity:int
      {
         mxREAL=0,
         mxCOMPLEX
      };
		
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern int mxGetM(IntPtr pa);	

      /* 
       * Set row dimension
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxSetM( IntPtr pa, int m );


      /* 
       * Get column dimension
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern int mxGetN(
         IntPtr pa		/* pointer to array */
         );
      /* 
       * Set column dimension
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxSetN(IntPtr pa, int n);

      /*
       * Is array empty
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsEmpty(
         IntPtr pa		/* pointer to array */
         );

      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern IntPtr mxGetPr(IntPtr pa);  //create an unsafe method for copying this into a double array?
	

      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern IntPtr mxCreateDoubleMatrix(int m, int n, mxComplexity flag);

      /*
       * mxArray destructor
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxDestroyArray(IntPtr pa);
      //
      //		
      //

      /*
       * Copies characters from a MATLAB array to a char array
       * nChars is the number of bytes in the output buffer
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxGetNChars(IntPtr pa, String buf, int nChars);

      /* 
       * Converts a string array to a C-style string.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern int mxGetString(IntPtr pa, String buf, int buflen);

      /*
       * Create a 1-by-n string array initialized to null terminated string
       * where n is the length of the string.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern IntPtr mxCreateString(String str);

      /*
       * Create a 1-by-n string array initialized to str.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern IntPtr mxCreateStringFromNChars(String str, int n);


      /*
       * Return the class (catergory) of data that the array holds.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern mxClassID mxGetClassID(IntPtr pa);
		
      /*
       * Create a 2-Dimensional cell array, with each cell initialized
       * to NULL.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern IntPtr mxCreateCellMatrix(int m, int n);

      /*
       * Make a deep copy of an array, return a pointer to the copy. 
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern IntPtr mxDuplicateArray(IntPtr pa);

      /*
       * Set an element in a cell array to a copy of the specified value.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxSetCopyInCell(IntPtr pa, int i, IntPtr value);
		
      /*
       * Function for obtaining MATLAB's concept of EPS
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern double mxGetEps();


      /*
       * Function for obtaining MATLAB's concept of INF (Used in MEX-File callback).
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern double mxGetInf();


      /*
       * Function for obtaining MATLAB's concept of NaN (Used in MEX-File callback).
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern double mxGetNaN();


      /*
       * test for finiteness in a machine-independent manner
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsFinite(
         double x                  /* value to test */
         );


      /*
       * test for infinity in a machine-independent manner
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsInf(
         double x                  /* value to test */
         );


      /*
       * test for NaN in a machine-independent manner
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsNaN(
         double x                  /* value to test */
         );

      /* 
       * Return the name of an array's class.  
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern String mxGetClassName(IntPtr pa);


      /*
       * Determine whether an array is a member of the specified class. 
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsClass(IntPtr pa, String name);

      /*
       * Set an element in a cell array to the specified value.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxSetCell(IntPtr pa, int i, IntPtr value);

      /*
       * Get a pointer to the specified cell element. 
       */ 
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern IntPtr mxGetCell(IntPtr pa, int i);

      /* 
       * Get number of elements in array
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern int mxGetNumberOfElements(	
         IntPtr pa		/* pointer to array */
         );

      /*
      * Set imaginary data pointer for numeric array
      */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxSetPi(
         IntPtr pa,		/* pointer to mxArray */
         IntPtr pi			/* imaginary data array pointer (double *)*/
         );


      /*
       * Determine whether the given array contains complex data.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsComplex(IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * double-precision floating-point numbers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsDouble(IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * single-precision floating-point numbers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsSingle(IntPtr pa);


      /*
       * Determine whether the given array's logical flag is on.
       */ 
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsLogical(IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * signed 8-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsInt8(IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * unsigned 8-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsUint8(IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * signed 16-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsInt16(IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * unsigned 16-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsUint16( IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * signed 32-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsInt32( IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * unsigned 32-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsUint32( IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * signed 64-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsInt64( IntPtr pa);


      /*
       * Determine whether the specified array represents its data as 
       * unsigned 64-bit integers.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsUint64( IntPtr pa);

      /*
       * Get imaginary data pointer for numeric array
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern double[] mxGetPi(
         IntPtr pa		/* pointer to array */
         );

      /*
       * Set real data pointer for numeric array
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxSetPr(
         IntPtr pa,		/* pointer to Mxarray */
         IntPtr pr			/* real data array pointer (double)*/
         );


      /* 
       * Determine whether the specified array contains numeric (as opposed 
       * to cell or struct) data.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsNumeric( IntPtr pa);


      /* 
       * Determine whether the given array is a cell array.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsCell( IntPtr pa);


      /*  
       * Determine whether the given array contains character data. 
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsChar( IntPtr pa);


      /*
       * Determine whether the given array is a sparse (as opposed to full). 
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsSparse( IntPtr pa);


      /*
       * Determine whether the given array is a structure array.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsStruct( IntPtr pa);


      /*
       * Determine whether the given array is an opaque array.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsOpaque( IntPtr pa);


      /*
       * Returns true if specified array is a function object.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern bool mxIsFunctionHandle( IntPtr pa);

      /* 
       * Set array name.  This routine copies the string pointed to by s
       * into the mxMAXNAM length character name field.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern void mxSetName(
         IntPtr pa,		/* pointer to array */
         String s		/* string to copy into name */
         );

      /* 
       * Get pointer to array name.
       */
      [ DllImport( "libmx.dll", CallingConvention=CallingConvention.Cdecl)]
      public static extern String mxGetName(
         IntPtr pa		/* pointer to array */
         );

      //
      //
      //
      //


      //Return a jagged double system.array type given an mxArray pointer
      public static double[][] getDoubleMatrix(IntPtr pa)
      {
         int intM;
         int intN;
         int intCnt;
         double[][] dblSystemArray;
         double[] dblTmpSystemArray;

         IntPtr intPtrDblArray;

         intM = mxGetM(pa);
         intN = mxGetN(pa);

         if(intM ==0 || intN ==0)
            return null;

         dblSystemArray=new double[intM][];
         dblTmpSystemArray = new Double[intM * intN];
			
         intPtrDblArray = mxGetPr(pa);

         Marshal.Copy(intPtrDblArray,dblTmpSystemArray,0,intN*intM);

         //Alloc and copy the buffer.
         for(intCnt=0;intCnt<intM;intCnt++)
         {
            dblSystemArray[intCnt]=new Double[intN];
            for(int intColCnt=0;intColCnt<intN;intColCnt++) 
            {
               dblSystemArray[intCnt][intColCnt]=dblTmpSystemArray[intColCnt*intM + intCnt];
            }
         }
			
         return dblSystemArray;
      }


      //Return the square multidimensional array
      public static double[,] getDoubleMatrixM(IntPtr pa)
      {

         double[][] dblTmpJagged = getDoubleMatrix(pa);
         int intRows=dblTmpJagged.Length;
         int intCols = dblTmpJagged[0].Length;

         double[,] dblTmpArray = new double[dblTmpJagged.Length,dblTmpJagged[0].Length];

         for(int i=0;i<intRows;i++)
            for(int j=0;j<intCols;j++)
               dblTmpArray[i,j]=dblTmpJagged[i][j];


         return(dblTmpArray);

      }

      public static String getStringMatrix(IntPtr pa)
      {
         int intM;
         int intN;
         
         string strSystemString;
         IntPtr intPtrStrArray;

         if(mxMatrix.mxGetClassID(pa) != mxMatrix.mxClassID.mxCHAR_CLASS)
         {
            //oops.
            return(null);
         }

         intM = mxGetM(pa);
         intN = mxGetN(pa);

         if(intM ==0 || intN ==0)
            return null;

         intPtrStrArray = mxGetPr(pa);

         strSystemString=Marshal.PtrToStringAuto(intPtrStrArray,intN*intM);

         return strSystemString;

      }


      //build an mxArray from a jagged 2-d double System.array type.
      public static IntPtr getMxArray(double[][] dblSystemArray)
      {
         double[] dblTmpArray;
         IntPtr mxPtr=(IntPtr)0;
         IntPtr mxPtrTmp=(IntPtr)0;
         IntPtr ptrDataTmp;

         int intRows=dblSystemArray.Length;
         int intCols=dblSystemArray[0].Length;

         //Create the matrix
         try 
         {
            mxPtrTmp=mxCreateDoubleMatrix(intRows,intCols,mxMatrix.mxComplexity.mxREAL);
         }
         catch
         {
            //uhoh
            return (IntPtr)0;
         }

         //Now, fill it up
         dblTmpArray = new double[intRows * intCols];

         //Matlab stores things column wise
         for(int intRowCnt=0;intRowCnt<intRows;intRowCnt++)
            for(int intColCnt=0;intColCnt<intCols;intColCnt++)
            {
               dblTmpArray[intColCnt * intRows + intRowCnt]=dblSystemArray[intRowCnt][intColCnt];
            }

         //This memory must be released using FreeHGlobal
         ptrDataTmp = Marshal.AllocHGlobal(dblTmpArray.Length * Marshal.SizeOf(dblTmpArray[0]));
         Marshal.Copy(dblTmpArray,0,ptrDataTmp,dblTmpArray.Length);
         mxSetPr(mxPtrTmp, ptrDataTmp);

         //Let matlab handle the garbage collection with a whole object.  We can nuke it later with mxDestroyArray
         mxPtr=mxDuplicateArray(mxPtrTmp);
         Marshal.FreeHGlobal(ptrDataTmp);

         return mxPtr;
      }

      public static IntPtr getMxArray(double[,] dblSystemArray)
      {
         double[][] dblTmpArray;
         int intCols=dblSystemArray.GetUpperBound(1)-dblSystemArray.GetLowerBound(1)+1;
         int intRows=dblSystemArray.GetUpperBound(0)-dblSystemArray.GetLowerBound(0)+1;

         dblTmpArray=new double[intRows][];
         for(int i=0;i<intRows;i++)
            dblTmpArray[i]=new double[intCols];

         for(int i=0;i<intRows;i++)
            for(int j=0;j<intCols;j++)
               dblTmpArray[i][j]=dblSystemArray[i,j];

         return getMxArray(dblTmpArray);
      }

      public static IntPtr getMxArray(double dblSystemDouble)
      {
         double[][] dblTmpArray=new double[1][];
         dblTmpArray[0]=new double[1];

         dblTmpArray[0][0]=dblSystemDouble;

         return getMxArray(dblTmpArray);
      }	

      public mxMatrix()
      {
      }
   }
}
