using System;
using System.Runtime.InteropServices;
using matlabMatrix;

namespace RTGA
{
	/// <summary>
	/// Class implementing access to the functions written in Matlab
	/// Mihai Virtosu, virtosumihai@yahoo.com
	/// </summary>
	public class multarg
	{
		public multarg()
		{
			//
			// TODO: Add constructor logic here
			//
			//Initialize the matlab engine.
			libPidOpt2Initialize();
		}
		//probably have to explicitly call this
		public void cleanup()
		{
			libPidOpt2Terminate();
		}


		// Matlab imports
		// function [CarlsonD, CarlsonDIndex, CarlsonC, CarlsonX, CarlsonIndices, Cj, status] = computeCarlson(dataMatrix)
		/* from the header file:
		 * extern mxArray * mlfMultarg(mxArray * * b, mxArray * x, mxArray * y);
		 * extern void libMultArgInitialize(void);
		 * extern void libMultArgTerminate(void);
		*/
		//One of the return matrices is in the argument list
		[ DllImport( "libPidOpt2.dll") ]
		public static extern IntPtr mlfPid_opt2(ref IntPtr suprareglaj,
			ref IntPtr Kr,
			ref IntPtr Ti,
			ref IntPtr Td,
			ref IntPtr Chrom,
			ref IntPtr y,
			ref IntPtr t,
			ref IntPtr yKessler,
			ref IntPtr tKessler,
			IntPtr a,
			IntPtr b,
			IntPtr c,
			IntPtr Kp,
			IntPtr MAXGEN,
			IntPtr Chrom_in,
			IntPtr CSreg,
			IntPtr CTt,
			IntPtr minKr,
			IntPtr minTi,
			IntPtr minTd,
			IntPtr maxKr,
			IntPtr maxTi,
			IntPtr maxTd,
			IntPtr inKr,
			IntPtr inTi,
			IntPtr inTd);

		[ DllImport( "libPidOpt2.dll" ) ]
		public static extern void libPidOpt2Initialize();

		[ DllImport( "libPidOpt2.dll") ]
		public static extern void libPidOpt2Terminate();

		//The public function usable by c# clients.  Some massaging might be necessary to make a usable
		//com interface for vb clients.
		public void csMultArg(ref double[,] tt, 
			ref double[,] suprareglaj,
			ref double[,] Kr,
			ref double[,] Ti,
			ref double[,] Td,
			ref double[,] Chrom,
			ref double[,] y,
			ref double[,] t,
			ref double[,] yKessler,
			ref double[,] tKessler,
			double[,] a,
			double[,] b,
			double[,] c,
			double[,] Kp,
			double[,] MAXGEN,
			double[,] Chrom_in,
			double[,] CSreg,
			double[,] CTt,
			double[,] minKr,
			double[,] minTi,
			double[,] minTd,
			double[,] maxKr,
			double[,] maxTi,
			double[,] maxTd,
			double[,] inKr,
			double[,] inTi,
			double[,] inTd
			)
		{
			IntPtr mxtt=(IntPtr)null;
			IntPtr mxsuprareglaj=(IntPtr)null;
			IntPtr mxKr=(IntPtr)null;
			IntPtr mxTi=(IntPtr)null;
			IntPtr mxTd=(IntPtr)null;
			IntPtr mxt=(IntPtr)null;
			IntPtr mxChrom=(IntPtr)null;
			IntPtr mxa=(IntPtr)null;
			IntPtr mxb=(IntPtr)null;
			IntPtr mxc=(IntPtr)null;
			IntPtr mxKp=(IntPtr)null;
			IntPtr mxMAXGEN=(IntPtr)null;
			IntPtr mxChrom_in=(IntPtr)null;
			IntPtr mxy=(IntPtr)null;
			IntPtr mxCSreg=(IntPtr)null;
			IntPtr mxCTt=(IntPtr)null;
			IntPtr mxyKessler=(IntPtr)null;
			IntPtr mxtKessler=(IntPtr)null;
			IntPtr mxminKr=(IntPtr)null;
			IntPtr mxminTi=(IntPtr)null;
			IntPtr mxminTd=(IntPtr)null;
			IntPtr mxmaxKr=(IntPtr)null;
			IntPtr mxmaxTi=(IntPtr)null;
			IntPtr mxmaxTd=(IntPtr)null;
			IntPtr mxinKr=(IntPtr)null;
			IntPtr mxinTi=(IntPtr)null;
			IntPtr mxinTd=(IntPtr)null;

			//Get matlab mxMatrices from the double matrix arguments using the included library/translation class
			mxtt=mxMatrix.getMxArray(tt);
			mxsuprareglaj=mxMatrix.getMxArray(suprareglaj);
			mxKr=mxMatrix.getMxArray(Kr);
			mxTi=mxMatrix.getMxArray(Ti);
			mxt=mxMatrix.getMxArray(t);
			mxTd=mxMatrix.getMxArray(Td);
			mxChrom=mxMatrix.getMxArray(Chrom);
			mxa=mxMatrix.getMxArray(a);
			mxb=mxMatrix.getMxArray(b);
			mxc=mxMatrix.getMxArray(c);
			mxKp=mxMatrix.getMxArray(Kp);
			mxMAXGEN=mxMatrix.getMxArray(MAXGEN);
			mxChrom_in=mxMatrix.getMxArray(Chrom_in);
			mxy=mxMatrix.getMxArray(y);
			mxCSreg=mxMatrix.getMxArray(CSreg);
			mxCTt=mxMatrix.getMxArray(CTt);
			mxyKessler=mxMatrix.getMxArray(yKessler);
			mxtKessler=mxMatrix.getMxArray(tKessler);
			mxminKr=mxMatrix.getMxArray(minKr);
			mxminTi=mxMatrix.getMxArray(minTi);
			mxminTd=mxMatrix.getMxArray(minTd);
			mxmaxKr=mxMatrix.getMxArray(maxKr);
			mxmaxTi=mxMatrix.getMxArray(maxTi);
			mxmaxTd=mxMatrix.getMxArray(maxTd);
			mxinKr=mxMatrix.getMxArray(inKr);
			mxinTi=mxMatrix.getMxArray(inTi);
			mxinTd=mxMatrix.getMxArray(inTd);


			//Call the imported compiled matlab function
			mxtt=mlfPid_opt2(ref mxsuprareglaj,
				ref mxKr,
				ref mxTi,
				ref mxTd,
				ref mxChrom,
				ref mxy,
				ref mxt,
				ref mxyKessler,
				ref mxtKessler,
				mxa,
				mxb,
				mxc,
				mxKp,
				mxMAXGEN,
				mxChrom_in,
				mxCSreg,
				mxCTt,
				mxminKr,
				mxminTi,
				mxminTd,
				mxmaxKr,
				mxmaxTi,
				mxmaxTd,
				mxinKr,
				mxinTi,
				mxinTd
				);

			//Get square double matrix from mxMatrix return values
			tt=mxMatrix.getDoubleMatrixM(mxtt);
			suprareglaj=mxMatrix.getDoubleMatrixM(mxsuprareglaj);
			Kr=mxMatrix.getDoubleMatrixM(mxKr);
			Ti=mxMatrix.getDoubleMatrixM(mxTi);
			Td=mxMatrix.getDoubleMatrixM(mxTd);
			Chrom=mxMatrix.getDoubleMatrixM(mxChrom);
			y=mxMatrix.getDoubleMatrixM(mxy);
			t=mxMatrix.getDoubleMatrixM(mxt);
			CSreg=mxMatrix.getDoubleMatrixM(mxCSreg);
			CTt=mxMatrix.getDoubleMatrixM(mxCTt);
			yKessler=mxMatrix.getDoubleMatrixM(mxyKessler);
			tKessler=mxMatrix.getDoubleMatrixM(mxtKessler);
			minKr=mxMatrix.getDoubleMatrixM(mxminKr);
			minTi=mxMatrix.getDoubleMatrixM(mxminTi);
			minTd=mxMatrix.getDoubleMatrixM(mxminTd);
			maxKr=mxMatrix.getDoubleMatrixM(mxmaxKr);
			maxTi=mxMatrix.getDoubleMatrixM(mxmaxTi);
			maxTd=mxMatrix.getDoubleMatrixM(mxmaxTd);
			inKr=mxMatrix.getDoubleMatrixM(mxinKr);
			inTi=mxMatrix.getDoubleMatrixM(mxinTi);
			inTd=mxMatrix.getDoubleMatrixM(mxinTd);

		}
	}
}
